@@ -1,7 +1,7 @@
 /**************************************************************************
 * This file is part of the WebIssues Desktop Client program
 * Copyright (C) 2006 Michał Męciński
-* Copyright (C) 2007-2015 WebIssues Team
+* Copyright (C) 2007-2017 WebIssues Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -291,6 +291,7 @@ public:
     static UserEntity find( int id );
 
     static QList<UserEntity> list();
+    static QList<UserEntity> visible();
 
     static bool exists( const QString& login, const QString& name );
 
@@ -516,7 +517,9 @@ public:
     QString name() const;
     DefinitionInfo definition() const;
 
+    const QString& fromProject() const;
     const QString& fromFolder() const;
+    const QString& toProject() const;
     const QString& toFolder() const;
 
     CommentEntity comment() const;
