@@ -1,7 +1,7 @@
 /**************************************************************************
 * This file is part of the WebIssues Desktop Client program
 * Copyright (C) 2006 Michał Męciński
-* Copyright (C) 2007-2015 WebIssues Team
+* Copyright (C) 2007-2017 WebIssues Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -277,7 +277,9 @@ public:
     int m_typeId;
     int m_attributeId;
 
+    QString m_fromProject;
     QString m_fromFolder;
+    QString m_toProject;
     QString m_toFolder;
 
     QString m_commentText;
