@@ -1,7 +1,7 @@
 /**************************************************************************
 * This file is part of the WebIssues Desktop Client program
 * Copyright (C) 2006 Michał Męciński
-* Copyright (C) 2007-2015 WebIssues Team
+* Copyright (C) 2007-2017 WebIssues Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -576,7 +576,11 @@ QStringList ViewDialog::userItems()
 
     items.append( QString( "[%1]" ).arg( tr( "Me" ) ) );
 
-    QList<UserEntity> users = UserEntity::list();
+    QList<UserEntity> users;
+    if ( dataManager->currentUserAccess() == AdminAccess )
+        users = UserEntity::list();
+    else
+        users = UserEntity::visible();
 
     foreach ( const UserEntity& user, users ) {
         if ( user.access() == NoAccess )
