@@ -1,7 +1,7 @@
 /**************************************************************************
 * This file is part of the WebIssues Desktop Client program
 * Copyright (C) 2006 Michał Męciński
-* Copyright (C) 2007-2015 WebIssues Team
+* Copyright (C) 2007-2017 WebIssues Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -321,10 +321,12 @@ HtmlText IssueDetailsGenerator::formatChange( const ChangeEntity& change, HtmlTe
             result.appendText( tr( "Issue moved from" ) );
             result.appendText( " " );
 
-            if ( change.fromFolder().isEmpty() ) {
+            if ( change.fromProject().isEmpty() ) {
                 result.appendText( tr( "Unknown Folder" ) );
             } else {
                 result.appendText( "\"" );
+                result.appendText( change.fromProject() );
+                result.appendText( QString::fromUtf8( " — " ) );
                 result.appendText( change.fromFolder() );
                 result.appendText( "\"" );
             }
@@ -333,10 +335,12 @@ HtmlText IssueDetailsGenerator::formatChange( const ChangeEntity& change, HtmlTe
             result.appendText( tr( "to" ) );
             result.appendText( " " );
 
-            if ( change.toFolder().isEmpty() ) {
+            if ( change.toProject().isEmpty() ) {
                 result.appendText( tr( "Unknown Folder" ) );
             } else {
                 result.appendText( "\"" );
+                result.appendText( change.toProject() );
+                result.appendText( QString::fromUtf8( " — " ) );
                 result.appendText( change.toFolder() );
                 result.appendText( "\"" );
             }
