@@ -1,7 +1,7 @@
 /**************************************************************************
 * This file is part of the WebIssues Desktop Client program
 * Copyright (C) 2006 Michał Męciński
-* Copyright (C) 2007-2015 WebIssues Team
+* Copyright (C) 2007-2017 WebIssues Team
 *
 * This program is free software: you can redistribute it and/or modify
 * it under the terms of the GNU General Public License as published by
@@ -32,6 +32,7 @@ QueryGenerator::QueryGenerator() :
     m_folderId( 0 ),
     m_typeId( 0 ),
     m_viewId( 0 ),
+    m_projectId( 0 ),
     m_searchColumn( -1 ),
     m_sortColumn( -1 ),
     m_sortOrder( Qt::AscendingOrder ),
@@ -102,6 +103,11 @@ void QueryGenerator::initializeCommon( bool withLocation )
     m_sortOrder = order.second;
 }
 
+void QueryGenerator::setProject( int projectId )
+{
+    m_projectId = projectId;
+}
+
 void QueryGenerator::setSearchText( int column, const QString& text )
 {
     m_searchColumn = column;
@@ -243,6 +249,11 @@ QString QueryGenerator::generateConditions()
         m_arguments.append( m_typeId );
     }
 
+    if ( m_folderId == 0 && m_projectId != 0 ) {
+        conditions.append( "f.project_id = ?" );
+        m_arguments.append( m_projectId );
+    }
+
     IssueTypeCache* cache = dataManager->issueTypeCache( m_typeId );
 
     QList<DefinitionInfo> allFilters = m_filters;
