@@ -1,9 +1,9 @@
 /**************************************************************************
-* Extensible SQLite driver for Qt4
-* Copyright (C) 2011-2012 Michał Męciński
+* Extensible SQLite driver for Qt
+* Copyright (C) 2011-2015 Michał Męciński
 *
 * This library is free software: you can redistribute it and/or modify
-* it under the terms of the GNU Lesser General Public License version 2.1
+* it under the terms of the GNU Lesser General Public License version 3
 * as published by the Free Software Foundation.
 *
 * This library is distributed in the hope that it will be useful,
@@ -15,14 +15,16 @@
 * along with this library.  If not, see <http://www.gnu.org/licenses/>.
 *
 * This library is based on the QtSql module of the Qt Toolkit
-* Copyright (C) 2012 Nokia Corporation and/or its subsidiary(-ies).
+* Copyright (C) 2014 Digia Plc and/or its subsidiary(-ies).
 **************************************************************************/
 
 #include "sqlitedriver.h"
 
+#include "sqlcachedresult.h"
 #include "sqliteextension.h"
 
 #include <qcoreapplication.h>
+#include <qdatetime.h>
 #include <qvariant.h>
 #include <qsqlerror.h>
 #include <qsqlfield.h>
@@ -31,7 +33,6 @@
 #include <qstringlist.h>
 #include <qvector.h>
 #include <qdebug.h>
-#include <qsqldriverplugin.h>
 
 #if defined Q_OS_WIN
 # include <qt_windows.h>
@@ -45,7 +46,10 @@
 # include "sqlite3.h"
 #endif
 
+Q_DECLARE_OPAQUE_POINTER(sqlite3*)
 Q_DECLARE_METATYPE(sqlite3*)
+
+Q_DECLARE_OPAQUE_POINTER(sqlite3_stmt*)
 Q_DECLARE_METATYPE(sqlite3_stmt*)
 
 static QString _q_escapeIdentifier(const QString &identifier) 
@@ -68,31 +72,23 @@ static QVariant::Type qGetColumnType(const QString &tpName)
         return QVariant::Int;
     if (typeName == QLatin1String("double")
         || typeName == QLatin1String("float")
+        || typeName == QLatin1String("real")
         || typeName.startsWith(QLatin1String("numeric")))
         return QVariant::Double;
     if (typeName == QLatin1String("blob"))
         return QVariant::ByteArray;
+    if (typeName == QLatin1String("boolean")
+        || typeName == QLatin1String("bool"))
+        return QVariant::Bool;
     return QVariant::String;
 }
 
-static QString stringFromUnicode(const QChar* unicode)
-{
-#if (QT_VERSION >= 0x040700)
-    return QString(unicode);
-#else
-    int size = 0;
-    while (unicode[size] != 0)
-        size++;
-    return QString(unicode, size);
-#endif
-}
-
 static QSqlError qMakeError(sqlite3 *access, const QString &descr, QSqlError::ErrorType type,
                             int errorCode = -1)
 {
     return QSqlError(descr,
-                     stringFromUnicode(reinterpret_cast<const QChar *>(sqlite3_errmsg16(access))),
-                     type, errorCode);
+                     QString(reinterpret_cast<const QChar *>(sqlite3_errmsg16(access))),
+                     type, QString::number(errorCode));
 }
 
 #if defined(SQLITEDRIVER_DEBUG)
@@ -104,14 +100,42 @@ static void trace( void* /*arg*/, const char* query )
 
 #endif
 
+class SQLiteResultPrivate;
+
+class SQLiteResult : public SqlCachedResult
+{
+    friend class SQLiteDriver;
+    friend class SQLiteResultPrivate;
+public:
+    explicit SQLiteResult(const SQLiteDriver* db);
+    ~SQLiteResult();
+    QVariant handle() const;
+
+protected:
+    bool gotoNext(SqlCachedResult::ValueCache& row, int idx);
+    bool reset(const QString &query);
+    bool prepare(const QString &query);
+    bool exec();
+    int size();
+    int numRowsAffected();
+    QVariant lastInsertId() const;
+    QSqlRecord record() const;
+    void detachFromResultSet();
+    void virtual_hook(int id, void *data);
+    void setLastError(const QSqlError& e);
+
+private:
+    SQLiteResultPrivate* d;
+};
+
 class SQLiteDriverPrivate
 {
 public:
     inline SQLiteDriverPrivate() : access(0) {}
     sqlite3 *access;
+    QList <SQLiteResult *> results;
 };
 
-
 class SQLiteResultPrivate
 {
 public:
@@ -167,19 +191,43 @@ void SQLiteResultPrivate::initColumns(bool emptyResultset)
     q->init(nCols);
 
     for (int i = 0; i < nCols; ++i) {
-        QString colName = stringFromUnicode(reinterpret_cast<const QChar *>(
+        QString colName = QString(reinterpret_cast<const QChar *>(
                     sqlite3_column_name16(stmt, i))
                     ).remove(QLatin1Char('"'));
 
         // must use typeName for resolving the type to match SQLiteDriver::record
-        QString typeName = stringFromUnicode(reinterpret_cast<const QChar *>(
+        QString typeName = QString(reinterpret_cast<const QChar *>(
                     sqlite3_column_decltype16(stmt, i)));
-
-        int dotIdx = colName.lastIndexOf(QLatin1Char('.'));
-        QSqlField fld(colName.mid(dotIdx == -1 ? 0 : dotIdx + 1), qGetColumnType(typeName));
-
         // sqlite3_column_type is documented to have undefined behavior if the result set is empty
         int stp = emptyResultset ? -1 : sqlite3_column_type(stmt, i);
+
+        QVariant::Type fieldType;
+
+        if (!typeName.isEmpty()) {
+            fieldType = qGetColumnType(typeName);
+        } else {
+            // Get the proper type for the field based on stp value
+            switch (stp) {
+            case SQLITE_INTEGER:
+                fieldType = QVariant::Int;
+                break;
+            case SQLITE_FLOAT:
+                fieldType = QVariant::Double;
+                break;
+            case SQLITE_BLOB:
+                fieldType = QVariant::ByteArray;
+                break;
+            case SQLITE_TEXT:
+                fieldType = QVariant::String;
+                break;
+            case SQLITE_NULL:
+            default:
+                fieldType = QVariant::Invalid;
+                break;
+            }
+        }
+
+        QSqlField fld(colName, fieldType);
         fld.setSqlType(stp);
         rInf.append(fld);
     }
@@ -232,7 +280,6 @@ bool SQLiteResultPrivate::fetchNext(SqlCachedResult::ValueCache &values, int idx
                 values[i + idx] = sqlite3_column_int64(stmt, i);
                 break;
             case SQLITE_FLOAT:
-#if ( QT_VERSION >= 0x040600 )
                 switch(q->numericalPrecisionPolicy()) {
                     case QSql::LowPrecisionInt32:
                         values[i + idx] = sqlite3_column_int(stmt, i);
@@ -246,9 +293,6 @@ bool SQLiteResultPrivate::fetchNext(SqlCachedResult::ValueCache &values, int idx
                         values[i + idx] = sqlite3_column_double(stmt, i);
                         break;
                 };
-#else
-                values[i + idx] = sqlite3_column_double(stmt, i);
-#endif
                 break;
             case SQLITE_NULL:
                 values[i + idx] = QVariant(QVariant::String);
@@ -295,24 +339,21 @@ SQLiteResult::SQLiteResult(const SQLiteDriver* db)
 {
     d = new SQLiteResultPrivate(this);
     d->access = db->d->access;
+    db->d->results.append(this);
 }
 
 SQLiteResult::~SQLiteResult()
 {
+    const QSqlDriver *sqlDriver = driver();
+    if (sqlDriver)
+        const_cast<SQLiteDriverPrivate*>(qobject_cast<const SQLiteDriver *>(sqlDriver)->d)->results.removeOne(this);
     d->cleanup();
     delete d;
 }
 
 void SQLiteResult::virtual_hook(int id, void *data)
 {
-    switch (id) {
-    case QSqlResult::DetachFromResultSet:
-        if (d->stmt)
-            sqlite3_reset(d->stmt);
-        break;
-    default:
-        SqlCachedResult::virtual_hook(id, data);
-    }
+    SqlCachedResult::virtual_hook(id, data);
 }
 
 bool SQLiteResult::reset(const QString &query)
@@ -349,7 +390,7 @@ bool SQLiteResult::prepare(const QString &query)
                      "Unable to execute statement"), QSqlError::StatementError, res));
         d->finalize();
         return false;
-    } else if (pzTail && !stringFromUnicode(reinterpret_cast<const QChar *>(pzTail)).trimmed().isEmpty()) {
+    } else if (pzTail && !QString(reinterpret_cast<const QChar *>(pzTail)).trimmed().isEmpty()) {
         setLastError(qMakeError(d->access, QCoreApplication::translate("SQLiteResult",
             "Unable to execute multiple statements at a time"), QSqlError::StatementError, SQLITE_MISUSE));
         d->finalize();
@@ -391,6 +432,7 @@ bool SQLiteResult::exec()
                                             ba->size(), SQLITE_STATIC);
                     break; }
                 case QVariant::Int:
+                case QVariant::Bool:
                     res = sqlite3_bind_int(d->stmt, i + 1, value.toInt());
                     break;
                 case QVariant::Double:
@@ -400,6 +442,20 @@ bool SQLiteResult::exec()
                 case QVariant::LongLong:
                     res = sqlite3_bind_int64(d->stmt, i + 1, value.toLongLong());
                     break;
+                case QVariant::DateTime: {
+                    const QDateTime dateTime = value.toDateTime();
+                    const QString str = dateTime.toString(QStringLiteral("yyyy-MM-ddThh:mm:ss.zzz"));
+                    res = sqlite3_bind_text16(d->stmt, i + 1, str.utf16(),
+                                              str.size() * sizeof(ushort), SQLITE_TRANSIENT);
+                    break;
+                }
+                case QVariant::Time: {
+                    const QTime time = value.toTime();
+                    const QString str = time.toString(QStringLiteral("hh:mm:ss.zzz"));
+                    res = sqlite3_bind_text16(d->stmt, i + 1, str.utf16(),
+                                              str.size() * sizeof(ushort), SQLITE_TRANSIENT);
+                    break;
+                }
                 case QVariant::String: {
                     // lifetime of string == lifetime of its qvariant
                     const QString *str = static_cast<const QString*>(value.constData());
@@ -469,6 +525,12 @@ QSqlRecord SQLiteResult::record() const
     return d->rInf;
 }
 
+void SQLiteResult::detachFromResultSet()
+{
+    if (d->stmt)
+        sqlite3_reset(d->stmt);
+}
+
 QVariant SQLiteResult::handle() const
 {
     return qVariantFromValue(d->stmt);
@@ -524,6 +586,7 @@ bool SQLiteDriver::hasFeature(DriverFeature f) const
     case BatchOperations:
     case EventNotifications:
     case MultipleResultSets:
+    case CancelQuery:
         return false;
     }
     return false;
@@ -551,6 +614,11 @@ bool SQLiteDriver::open(const QString & db, const QString &, const QString &, co
         installSQLiteExtension(d->access);
         return true;
     } else {
+        if (d->access) {
+            sqlite3_close(d->access);
+            d->access = 0;
+        }
+
         setLastError(qMakeError(d->access, tr("Error opening database"),
                      QSqlError::ConnectionError));
         setOpenError(true);
@@ -561,6 +629,10 @@ bool SQLiteDriver::open(const QString & db, const QString &, const QString &, co
 void SQLiteDriver::close()
 {
     if (isOpen()) {
+        foreach (SQLiteResult *result, d->results) {
+            result->d->finalize();
+        }
+
         if (sqlite3_close(d->access) != SQLITE_OK)
             setLastError(qMakeError(d->access, tr("Error closing database"),
                                     QSqlError::ConnectionError));
@@ -688,10 +760,8 @@ QSqlIndex SQLiteDriver::primaryIndex(const QString &tblname) const
         return QSqlIndex();
 
     QString table = tblname;
-#if (QT_VERSION >= 0x040500)
     if (isIdentifierEscaped(table, QSqlDriver::TableName))
         table = stripDelimiters(table, QSqlDriver::TableName);
-#endif
 
     QSqlQuery q(createResult());
     q.setForwardOnly(true);
@@ -704,10 +774,8 @@ QSqlRecord SQLiteDriver::record(const QString &tbl) const
         return QSqlRecord();
 
     QString table = tbl;
-#if (QT_VERSION >= 0x040500)
     if (isIdentifierEscaped(table, QSqlDriver::TableName))
         table = stripDelimiters(table, QSqlDriver::TableName);
-#endif
 
     QSqlQuery q(createResult());
     q.setForwardOnly(true);
@@ -733,52 +801,3 @@ void SQLiteDriver::setLastError(const QSqlError& e)
 #endif
     QSqlDriver::setLastError(e);
 }
-
-class SQLiteDriverPlugin : public QSqlDriverPlugin
-{
-public:
-    SQLiteDriverPlugin();
-
-    QSqlDriver* create(const QString &);
-    QStringList keys() const;
-};
-
-SQLiteDriverPlugin::SQLiteDriverPlugin()
-    : QSqlDriverPlugin()
-{
-}
-
-QSqlDriver* SQLiteDriverPlugin::create(const QString &name)
-{
-    if (name == QLatin1String("SQLITEX")) {
-        SQLiteDriver* driver = new SQLiteDriver();
-        return driver;
-    }
-    return 0;
-}
-
-QStringList SQLiteDriverPlugin::keys() const
-{
-    QStringList l;
-    l << QLatin1String("SQLITEX");
-    return l;
-}
-
-#if defined(SQLITEDRIVER_EXPORT)
-
-Q_EXPORT_STATIC_PLUGIN(SQLiteDriverPlugin)
-Q_EXPORT_PLUGIN2(sqlitex, SQLiteDriverPlugin)
-
-#else
-
-QObject* qt_plugin_instance_sqlitex()
-{
-    static QPointer<QObject> instance;
-    if (!instance)
-        instance = new SQLiteDriverPlugin();
-    return instance;
-}
-
-Q_IMPORT_PLUGIN(sqlitex)
-
-#endif
